"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.drawQuery = exports.durationToString = exports.colors = void 0;
const chalk_1 = __importDefault(require("chalk"));
exports.colors = [
    '#b54605',
    '#4a768f',
    '#a69890',
    '#e68d5a',
    '#93c7e6',
    '#d98dab',
    '#52806d',
    '#07e089',
    '#e80761',
    '#870c3d',
    '#4e2366',
    '#44732a',
    '#a2f277',
    '#a92feb',
    '#4e2a61',
];
const SYMBOLS = {
    Empty: '',
    Full: '█',
    SevenEighths: '█',
    ThreeQuarters: '█',
    FiveEighths: '█',
    Half: '▌',
    ThreeEighths: '▌',
    Quarter: '▌',
    Eighth: '▌',
};
const getSymbolNormal = (value) => {
    if (value <= 0) {
        return SYMBOLS.Empty;
    }
    else if (value <= 1 / 8) {
        return SYMBOLS.Eighth;
    }
    else if (value <= 1 / 4) {
        return SYMBOLS.Quarter;
    }
    else if (value <= 3 / 8) {
        return SYMBOLS.ThreeEighths;
    }
    else if (value <= 1 / 2) {
        return SYMBOLS.Half;
    }
    else if (value <= 5 / 8) {
        return SYMBOLS.FiveEighths;
    }
    else if (value <= 3 / 4) {
        return SYMBOLS.ThreeQuarters;
    }
    else if (value <= 7 / 8) {
        return SYMBOLS.SevenEighths;
    }
    else {
        return SYMBOLS.Full;
    }
};
const splitNumberToParts = (value = 0) => {
    const [int, rest = '0'] = value.toString().split('.');
    return [parseInt(int, 10), parseInt(rest, 10) / Math.pow(10, rest.length)];
};
function durationToWidth(duration, maxDuration, maxWidth) {
    return (duration / maxDuration) * maxWidth;
}
function queryToString(query, colorByQueryType, maxWidth, maxDuration) {
    var _a;
    const width = durationToWidth(query.time_in_nanos, maxDuration, maxWidth);
    let [integer, rest] = splitNumberToParts(width);
    const color = (_a = colorByQueryType.get(query.type)) !== null && _a !== void 0 ? _a : '#DEADED';
    const padding = chalk_1.default.hex(color)(getSymbolNormal(1));
    const duration = durationToString(query.time_in_nanos);
    const label = integer > query.type.length + 2 + duration.length ? `${query.type} (${duration})` : query.type;
    let result = '';
    if (integer > label.length + 1) {
        result += `${chalk_1.default.bgHex(color)(label)}${padding}`;
        integer -= label.length + 1;
    }
    // shamelessly taken from
    // https://github.com/gribnoysup/wunderbar/blob/b42565b9af92addc82c53bea6e4512339822578d/lib/bars.js
    result += chalk_1.default.hex(color)(getSymbolNormal(integer).repeat(integer) + getSymbolNormal(integer === 0 && rest === 0 ? 0.001 : rest));
    return result;
}
function durationToString(durationInNanos) {
    const microSeconds = Math.round(durationInNanos / 1000);
    if (microSeconds > 1000) {
        const ms = Math.round(microSeconds / 1000);
        return `${ms} ms`;
    }
    if (microSeconds === 0) {
        return `${durationInNanos} ns`;
    }
    return `${microSeconds} µs`;
}
exports.durationToString = durationToString;
function drawQuery(query, colorByQueryType, output, maxDuration, offset = 0, depth = 1) {
    const maxWidth = output.columns;
    let maxDepth = depth;
    // Start by rendering the query itself
    const string = queryToString(query, colorByQueryType, maxWidth, maxDuration);
    output.cursorTo(Math.ceil(offset));
    output.write(`${string}`);
    output.cursorTo(Math.ceil(offset));
    // Recurse down into the children
    if (query.children !== undefined) {
        let childOffset = offset;
        // Move us one line deeper
        output.moveCursor(0, 1);
        for (const child of query.children) {
            maxDepth = Math.max(drawQuery(child, colorByQueryType, output, maxDuration, childOffset, depth + 1), maxDepth);
            childOffset += durationToWidth(child.time_in_nanos, maxDuration, maxWidth);
        }
        // Move us back to the original position we started at
        output.cursorTo(Math.ceil(offset));
        output.moveCursor(0, -1);
    }
    return maxDepth;
}
exports.drawQuery = drawQuery;
//# sourceMappingURL=draw.js.map